/**
 * This file is part of the SMT-LIB 2 preprocessor.
 * <http://www.cprover.org/SMT-LIB-LSM/>
 *
 * Copyright (C) 2009 Philipp Ruemmer <ph_r@gmx.net>
 *
 * Princess is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Princess is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Princess.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * The empty theory, providing propositional logic and standard operations
 * available on all types (equality, ite).
 */

#ifndef THEORY_EMPTY
#define THEORY_EMPTY 1

#include "theory.H"
#include "../smtlib-absyn.H"
#include "../smtlib-resolver.H"
#include "poly-function-family.H"

namespace Theories {

using namespace SMTLibAbsyn;

class Empty;

class TheoryHelpers : public Theory {
protected:
    void registerMonoFunction(Environment &env,
                              const MonoFunctionPtr& function) const;
    void registerPolyFunction(Environment &env,
                              const PolyFunctionFamilyPtr& function) const;

public:
    virtual const Empty& getEmpty() const = 0;
};

class Empty : public TheoryHelpers {
public:
    Empty();

    const SortCtorPtr           sort_Bool;
    const MonoFunctionPtr       fun_true;
    const MonoFunctionPtr       fun_false;
    const MonoFunctionPtr       fun_not;
    const MonoFunctionPtr       fun_implies;
    const MonoFunctionPtr       fun_and;
    const MonoFunctionPtr       fun_or;
    const MonoFunctionPtr       fun_xor;
    const PolyFunctionFamilyPtr polyfun_equal;
    const PolyFunctionFamilyPtr polyfun_ite;

    void registerSymbols(Environment &env) const;
    void registerSymbolsLocally(Environment &env) const;
    void visit(const FunctionAppPtr& node, FunctionVisitor& visitor) const;
    bool visitLocally(const FunctionAppPtr& node, FunctionVisitor& visitor) const;

    const Empty& getEmpty() const;
};

}

#endif
